"This program renders a TrueType font at a given size, collects the\n"
"rendered data in an array and generates a function to render a string\n"
"using the collected character graymap data.\n"
"\n"
"Its purpose is to embed a font into a program and to render with that\n"
"font without ever referring to freetype2 libraries or font files.\n"
"\n"
"In order to compile it, you\047ll need the freetype2 libraries and compile\n"
"as follows:\n"
"\n"
"  gcc -o bfont `pkg-config --cflags --libs freetype2` bfont.c -O3\n"
"\n"
"The O3 flag is useful for generating hash functions.\n"
"\n"
"Command line arguments to the program is as follows:\n"
"\n"
"bfont -f font_file -s size -h hash_func_name -r render_func_name\n"
"optional arguments:\n"
"   -u    generates the utf8 decoding function also.\n"
"   -U    same as -u but the function is declared as static.\n"
"   -g    generates a hash function with the given name for the set.\n"
"   -G    same as -g, but static function.\n"
"   -d dpi\n"
"         Default for this value is 96.\n"
"You can also use -R instead of -r, which again makes the rendering\n"
"function static.\n"
"\n"
"The program reads a list of Unicode character codes to put in the\n"
"map. This list is read from the standard input and contains decimal\n"
"or hexadecimal character codes. Each line should have either a single\n"
"character code or a pair of character codes. The latter case indicates\n"
"a range and all characters in the range will be included in the final\n"
"character map.\n"
"\n"
"Here is an example input file for the Turkish language character set:\n"
"\n"
"32 126\n"
"0xe7\n"
" 0x11f\n"
" 0x131\n"
" 0xf6\n"
" 0xfc\n"
" 0x15f\n"
" 0xc7\n"
"\n"
"0x11e\n"
" 0x130\n"
" 0xd6\n"
" 0xdc\n"
" 0x15e\n"
" 0x2192\n"
" 0xb0\n"
"\n"
"The output, printed on stdout, contains a function with the prototype:\n"
"\n"
"void render_font\n"
"        (uint32_t *str, size_t L, \n"
"        int *RW, int *RH, int *RA,\n"
"        unsigned char **Rdata);\n"
"\n"
"The name \047render_font\047 can be changed from the command line. The string \n"
"should be a wide string in Unicode. *RW,*RH and *RA contain width, height\n"
"and ascent of the graymap upon return. *Rdata contains one byte per pixel,\n"
"in row-major format, allocated using malloc().\n"
"\n"
"This function uses a hash function to convert Unicode characters to glyph indices for its table. This hash function should have the prototype:\n"
"\n"
"  unsigned int fhash(unsigned int V);\n"
"\n"
"It should return a sentinel value if the given character is not in the\n"
"map. This sentinel value is the number of characters in the map. The program\n"
"can generate this hash function as well, just append the -g switch.\n"
"\n"
"If your input is in utf-8, the program can also add some utf-8 to utf-32 \n"
"conversion code to the output. The generated function\047s prototype is:\n"
"\n"
"int utf8_decode_string\n"
"  (uint8_t *input, size_t ilen, uint32_t *output, size_t *olen);\n"
"\n"
"output array should have a size of at least ilen.\n"
"\n"
"So, to sum up, usage is:\n"
"\n"
"bfont -f font_file -s size -h hash_func_name -r render_func_name\n"
"optional arguments:\n"
"   -u    generates the utf8 decoding function also.\n"
"   -U    same as -u but the function is declared as static.\n"
"   -g    generates a hash function with the given name for the set.\n"
"   -G    same as -g, but static function.\n"
"   -d dpi\n"
"         Default for this value is 96.\n"
"You can also use -R instead of -r, which again makes the rendering\n"
"function static.\n"
"\n"

